/******************************************************************************
 * @file     main.c
 * @version  V1.00
 * @brief    A project template for M480 MCU.
 *
 * @copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#include <stdio.h>
#include "NuMicro.h"

/* Defines */
#define PLL_CLOCK           192000000
#define DELAY 							(PLL_CLOCK/5000)
#define LEFT 								0
#define RIGHT								1

/* Global variables */
uint32_t i; 
uint8_t a = LEFT;
uint16_t s = 500;

void GPC_IRQHandler(void)
{
    /* To check if PF.11 interrupt occurred */
    if(GPIO_GET_INT_FLAG(PC, BIT9))
    {
        GPIO_CLR_INT_FLAG(PC, BIT9);
				if (s == 50){
					s = 500;
				}
				else {
					s = s - 50;
				}
        printf("Flow speed is changed.\n");
    }
		else if (GPIO_GET_INT_FLAG(PC, BIT10))
    {
        GPIO_CLR_INT_FLAG(PC, BIT10);
				if (a == LEFT){
					a = RIGHT;
				}
				else {
					a = LEFT;
				}	
        printf("Flow direction is changed.\n");
    }
    else
    {
        /* Un-expected interrupt. Just clear all PB interrupts */
        PC->INTSRC = PC->INTSRC;
        printf("Un-expected interrupts.\n");
    }
}

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Set XT1_OUT(PF.2) and XT1_IN(PF.3) to input mode */
    PF->MODE &= ~(GPIO_MODE_MODE2_Msk | GPIO_MODE_MODE3_Msk);

    /* Enable External XTAL (4~24 MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HXTEN_Msk);

    /* Waiting for 12MHz clock ready */
    CLK_WaitClockReady(CLK_STATUS_HXTSTB_Msk);

    /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(PLL_CLOCK);
    /* Set PCLK0/PCLK1 to HCLK/2 */
    CLK->PCLKDIV = (CLK_PCLKDIV_PCLK0DIV2 | CLK_PCLKDIV_PCLK1DIV2);

    /* Enable UART clock */
    CLK_EnableModuleClock(UART0_MODULE);

    /* Select UART clock source from HXT */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UART0SEL_HXT, CLK_CLKDIV0_UART0(1));

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate SystemCoreClock. */
    SystemCoreClockUpdate();



    /* Set GPB multi-function pins for UART1 RXD and TXD */
    SYS->GPB_MFPH &= ~(SYS_GPB_MFPH_PB12MFP_Msk | SYS_GPB_MFPH_PB13MFP_Msk);
    SYS->GPB_MFPH |= (SYS_GPB_MFPH_PB12MFP_UART0_RXD | SYS_GPB_MFPH_PB13MFP_UART0_TXD);
		//SYS->GPB_MFPH &= ~(SYS_GPB_MFPL_PB2MFP_Msk | SYS_GPB_MFPL_PB3MFP_Msk);
		//SYS->GPB_MFPL |= (SYS_GPB_MFPL_PB2MFP_UART1_RXD | SYS_GPB_MFPL_PB3MFP_UART1_TXD);	
		
		/* Set LEDs pins as GPIO OUT Push-pull */
		PH->MODE &= ~(GPIO_MODE_MODE0_Msk | GPIO_MODE_MODE1_Msk | GPIO_MODE_MODE2_Msk);
		PH->MODE |= 1 << 0 | 1 << 2 | 1 << 4;	

	
    /* Lock protected registers */
    SYS_LockReg();
}

void Flow_Direction (uint8_t direction, uint16_t speed)
{
		if (direction == RIGHT){
			PH->DOUT &= ~(GPIO_DOUT_DOUT0_Msk);
			PH->DOUT |= GPIO_DOUT_DOUT1_Msk | GPIO_DOUT_DOUT2_Msk;			
			for (i=0; i<speed*DELAY; i++);
			
			PH->DOUT &= ~(GPIO_DOUT_DOUT1_Msk);
			PH->DOUT |= GPIO_DOUT_DOUT0_Msk | GPIO_DOUT_DOUT2_Msk;			
			for (i=0; i<speed*DELAY; i++);
			
			PH->DOUT &= ~(GPIO_DOUT_DOUT2_Msk);
			PH->DOUT |= GPIO_DOUT_DOUT1_Msk | GPIO_DOUT_DOUT0_Msk;			
			for (i=0; i<speed*DELAY; i++);
		}
		else {
			PH->DOUT &= ~(GPIO_DOUT_DOUT2_Msk);
			PH->DOUT |= GPIO_DOUT_DOUT1_Msk | GPIO_DOUT_DOUT0_Msk;			
			for (i=0; i<speed*DELAY; i++);
			
			PH->DOUT &= ~(GPIO_DOUT_DOUT1_Msk);
			PH->DOUT |= GPIO_DOUT_DOUT0_Msk | GPIO_DOUT_DOUT2_Msk;			
			for (i=0; i<speed*DELAY; i++);
			
			PH->DOUT &= ~(GPIO_DOUT_DOUT0_Msk);
			PH->DOUT |= GPIO_DOUT_DOUT1_Msk | GPIO_DOUT_DOUT2_Msk;			
			for (i=0; i<speed*DELAY; i++);
		}
}

/*
 * This is a template project for M480 series MCU. Users could based on this project to create their
 * own application without worry about the IAR/Keil project settings.
 *
 * This template application uses external crystal as HCLK source and configures UART0 to print out
 * "Hello World", users may need to do extra system configuration based on their system design.
 */

int main()
{
		SYS_Init();
    /* Init UART to 115200-8n1 for print message */
    UART_Open(UART0, 115200);
    /* Connect UART to PC, and open a terminal tool to receive following message */
   
		printf("\n\nCPU @ %d Hz\n", SystemCoreClock);
    printf("+------------------------------------------------------------+\n");
    printf("|                GPIO Interrupt Function Test                |\n");
    printf("+------------------------------------------------------------+\n\n");
	
		PH->DOUT |= GPIO_DOUT_DOUT0_Msk | GPIO_DOUT_DOUT1_Msk | GPIO_DOUT_DOUT2_Msk;
		
		/* Configure PC.10 (SW2) as Input mode and enable interrupt by falling edge trigger */
		GPIO_SetMode (PC, BIT10, GPIO_MODE_INPUT);
		GPIO_EnableInt (PC, 10, GPIO_INT_FALLING);
		NVIC_EnableIRQ (GPC_IRQn);
	
		/* Configure PC.9 (SW3) as Input mode and enable interrupt by falling edge trigger */
		GPIO_SetMode (PC, BIT9, GPIO_MODE_INPUT);
		GPIO_EnableInt (PC, 9, GPIO_INT_FALLING);
		NVIC_EnableIRQ (GPC_IRQn);
	
    /* Got no where to go, just loop forever */
    while(1)
		{
			Flow_Direction (a,s);
		}

}


/*** (C) COPYRIGHT 2016 Nuvoton Technology Corp. ***/
